import unittest
import numpy as np
from tests_configuration import *


class TestHumanCSVFileFormat(unittest.TestCase):

    # 1) test file name format: must have the form '<dataset_name>_stimuli.csv'
    def test_csv_name_suffix(self):
        csv_name = CFG.HUMAN_EXP_PATH.split('/')[-1]
        # must finish by '_human_experiments_data.csv'
        self.assertTrue(csv_name.endswith('_human_experimental_data.csv'))
        # At most one '_' in csv name, separating dataset name and stimuli
        self.assertEqual(len(csv_name.split('_')), 4)

    # 2) Test csv name prefix (same as the folder)
    def test_csv_name_prefix(self):
        csv_name = CFG.HUMAN_EXP_PATH.split('/').pop()
        dataset_name = CFG.HUMAN_EXP_PATH.split('/')[-3]
        csv_name_prefix = csv_name.split('_')[0]
        self.assertEqual(dataset_name, csv_name_prefix)

    # 3) Test key list
    def test_key_contents(self):

        # self.assertTrue(len(CFG.HUMAN_EXP_DATA.keys()[1:]) == 23)
        for key in CFG.HUMAN_EXP_DATA.keys():
            if key.startswith('Unnamed'):
                continue
            self.assertIn(key, ['subject_id', 'subject_language', 'triplet_id', 'TGT_item',
                                'OTH_item', 'X_item', 'TGT_first', 'user_ans',
                                'bin_user_ans', 'speaker_TGT', 'speaker_OTH', 'speaker_X',
                                'language_TGT', 'language_OTH', 'language_X', 'phone_TGT',
                                'phone_OTH', 'phone_X', 'context', 'prev_phone',
                                'next_phone', 'nb_stimuli', 'dataset'])

    # 4) Test each column format
    # 4) a) Test subject_id format
    def test_subject_id_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['subject_id'][i], str)

    # 4) b) Test subject_language format
    def test_subject_language_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['subject_language'][i], str)
            self.assertTrue(CFG.HUMAN_EXP_DATA['subject_language'][i].isupper())
            self.assertTrue(len(CFG.HUMAN_EXP_DATA['subject_language'][i]) == 2)

    # 4) c) Test triplet_id format
    def test_triplet_id_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['triplet_id'][i], str)
            self.assertTrue(CFG.HUMAN_EXP_DATA['triplet_id'][i].startswith('triplet_'))

    # 4) d) Test TGT_item format
    def test_TGT_item_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['TGT_item'][i], str)
            self.assertTrue(CFG.HUMAN_EXP_DATA['TGT_item'][i].endswith('.wav'))

    # 4) e) Test OTH_item format
    def test_OTH_item_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['OTH_item'][i], str)
            self.assertTrue(CFG.HUMAN_EXP_DATA['OTH_item'][i].endswith('.wav'))

    # 4) f) Test X_item format
    def test_X_item_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['X_item'][i], str)
            self.assertTrue(CFG.HUMAN_EXP_DATA['X_item'][i].endswith('.wav'))

    # 4) g) Test corr_ans format
    def test_corr_ans_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIn(CFG.HUMAN_EXP_DATA['TGT_first'][i], [True, False])

    # 4) h) Test user_ans format
    def test_user_ans_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIn(CFG.HUMAN_EXP_DATA['user_ans'][i], range(-3, 4))

    # 4) i) Test bin_user_ans format
    def test_bin_user_ans_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIn(CFG.HUMAN_EXP_DATA['bin_user_ans'][i], [-1, 1])

    # 4) j) Test speaker_TGT format
    def test_speaker_TGT_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['speaker_TGT'][i], str)

    # 4) k) Test speaker_OTH format
    def test_speaker_OTH_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['speaker_OTH'][i], str)

    # 4) l) Test speaker_X format
    def test_speaker_X_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['speaker_X'][i], str)

    # 4) m) Test language_TGT format
    def test_language_TGT_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['language_TGT'][i], str)
            self.assertTrue(CFG.HUMAN_EXP_DATA['language_TGT'][i].isupper())

    # 4) n) Test language_OTH format
    def test_language_OTH_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['language_OTH'][i], str)
            self.assertTrue(CFG.HUMAN_EXP_DATA['language_OTH'][i].isupper())

            # 4) o) Test language_X format ?
    def test_language_X_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['language_X'][i], str)
            self.assertTrue(CFG.HUMAN_EXP_DATA['language_X'][i].isupper())

    # 4) p) Test phone_TGT format (not in capital letter, IPA alphabet)
    def test_phone_TGT_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['phone_TGT'][i], str)
            self.assertIn(CFG.HUMAN_EXP_DATA['phone_TGT'][i], CFG.IPA)

    # 4) q) Test phone_OTH format (not in capital letter, IPA alphabet)
    def test_phone_OTH_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['phone_OTH'][i], str)
            self.assertIn(CFG.HUMAN_EXP_DATA['phone_OTH'][i], CFG.IPA)

    # 4) r) Test phone_X format (not in capital letter, IPA alphabet)
    def test_phone_X_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['phone_X'][i], str)
            self.assertIn(CFG.HUMAN_EXP_DATA['phone_X'][i], CFG.IPA)

    # 4) s) Test context format
    def test_context_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['context'][i], str)
            self.assertEqual(CFG.HUMAN_EXP_DATA['context'][i].split('_')[0], CFG.HUMAN_EXP_DATA['prev_phone'][i])
            self.assertEqual(CFG.HUMAN_EXP_DATA['context'][i].split('_')[1], CFG.HUMAN_EXP_DATA['next_phone'][i])

    # 4) t) Test prev_phone format
    def test_prev_phone_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['prev_phone'][i], str)
            self.assertIn(CFG.HUMAN_EXP_DATA['prev_phone'][i], CFG.IPA)

    # 4) u) Test next_phone format
    def test_next_phone_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.HUMAN_EXP_DATA['next_phone'][i], str)
            self.assertIn(CFG.HUMAN_EXP_DATA['next_phone'][i], CFG.IPA)

    # 4) v) Test nb_stimuli format
    def test_nb_stimuli_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertTrue(CFG.HUMAN_EXP_DATA['nb_stimuli'][i].item(), int)
            self.assertLess(0, CFG.HUMAN_EXP_DATA['nb_stimuli'][i])

    # 4) w) Test dataset format
    def test_dataset_key_format(self):
        random_indexes = np.random.randint(0, CFG.HUMAN_EXP_NB_ROWS, CFG.NB_TESTS)
        dataset_name = CFG.HUMAN_EXP_PATH.split('/')[-3]
        for i in random_indexes:
            self.assertEqual(CFG.HUMAN_EXP_DATA['dataset'][i], dataset_name)


if __name__ == '__main__':
    unittest.main()
    print('TESTS ON HUMAN EXPERIMENTS DATA FINISHED')
